@@ -6,7 +6,7 @@ use warnings;
 use Moose;
 use MooseX::Types::URI qw/ Uri /;
 
-our $VERSION = '0.41';
+our $VERSION = '0.42';
 
 # ABSTRACT: Net::SAML2::Binding::Redirect - HTTP Redirect binding for SAML
 
@@ -19,6 +19,7 @@ use URI::QueryParam;
 use Crypt::OpenSSL::RSA;
 use Crypt::OpenSSL::X509;
 use File::Slurp qw/ read_file /;
+use URI::Encode qw/uri_decode/;
 
 
 has 'key'   => (isa => 'Str', is => 'ro', required => 1);
@@ -26,6 +27,8 @@ has 'cert'  => (isa => 'Str', is => 'ro', required => 1);
 has 'url'   => (isa => Uri, is => 'ro', required => 1, coerce => 1);
 has 'param' => (isa => 'Str', is => 'ro', required => 1);
 has 'sig_hash' => (isa => 'Str', is => 'ro', required => 0);
+has 'sls_force_lcase_url_encoding'    => (isa => 'Bool', is => 'ro', required => 0);
+has 'sls_double_encoded_response' => (isa => 'Bool', is => 'ro', required => 0);
 
 
 sub sign {
@@ -83,6 +86,32 @@ sub verify {
     my $cert = Crypt::OpenSSL::X509->new_from_string($self->cert);
     my $rsa_pub = Crypt::OpenSSL::RSA->new_public_key($cert->pubkey);
 
+    my $signed;
+    my $saml_request;
+    my $sig = $u->query_param_delete('Signature');
+
+    # Some IdPs (PingIdentity) seem to double encode the LogoutResponse URL
+    if (defined $self->sls_double_encoded_response and $self->sls_double_encoded_response == 1) {
+        #if ($sigalg =~ m/%/) {
+        $signed = uri_decode($u->query);
+        $sig = uri_decode($sig);
+        $sigalg = uri_decode($sigalg);
+        $saml_request = uri_decode($u->query_param($self->param));
+    } else {
+        $signed = $u->query;
+        $saml_request = $u->query_param($self->param);
+    }
+
+    # What can we say about this one Microsoft Azure uses lower case in the
+    # URL encoding %2f not %2F.  As it is signed as %2f the resulting signed
+    # needs to change it to lowercase if the application layer reencoded the URL.
+    if (defined $self->sls_force_lcase_url_encoding and $self->sls_force_lcase_url_encoding == 1) {
+        # TODO: This is a hack.
+        $signed =~ s/(%..)/lc($1)/ge;
+    }
+
+    $sig = decode_base64($sig);
+
     if ($sigalg eq 'http://www.w3.org/2001/04/xmldsig-more#rsa-sha256') {
         $rsa_pub->use_sha256_hash;
     } elsif ($sigalg eq 'http://www.w3.org/2001/04/xmldsig-more#rsa-sha224') {
@@ -97,12 +126,10 @@ sub verify {
         die "Unsupported Signature Algorithim: $sigalg";
     }
 
-    my $sig = decode_base64($u->query_param_delete('Signature'));
-    my $signed = $u->query;
     die "bad sig" unless $rsa_pub->verify($signed, $sig);
 
     # unpack the SAML request
-    my $deflated = decode_base64($u->query_param($self->param));
+    my $deflated = decode_base64($saml_request);
     my $request = '';
     rawinflate \$deflated => \$request;
 
@@ -126,7 +153,7 @@ Net::SAML2::Binding::Redirect - Net::SAML2::Binding::Redirect - HTTP Redirect bi
 
 =head1 VERSION
 
-version 0.41
+version 0.42
 
 =head1 SYNOPSIS
 
@@ -186,6 +213,18 @@ sha1, sha224, sha256, sha384, sha512
 
 sha1 is current default but will change by version 44
 
+=item B<sls_force_lcase_url_encoding>
+
+Specifies that the IdP requires the encoding of a URL to be in lowercase.
+Necessary for a HTTP-Redirect of a LogoutResponse from Azure in particular.
+True (1) or False (0). Some web frameworks and underlying http requests assume
+that the encoding should be in the standard uppercase (%2F not %2f)
+
+=item B<sls_double_encoded_response>
+
+Specifies that the IdP response sent to the HTTP-Redirect is double encoded.
+The double encoding requires it to be decoded prior to processing.
+
 =back
 
 =head2 sign( $request, $relaystate )
