@@ -42,6 +42,15 @@ sub net_saml2_sp {
         org_display_name => 'Test',
         org_contact      => 'test@example.com',
         org_url          => 'http://www.example.com',
+        slo_url_soap     => '/slo-soap',
+        slo_url_redirect => '/sls-redirect-response',
+        slo_url_post     => '/sls-post-response',
+        acs_url_post     => '/consumer-post',
+        acs_url_artifact => '/consumer-artifact',
+        org_name         => 'Net::SAML2 Saml2Test',
+        org_display_name => 'Saml2Test app for Net::SAML2',
+        org_contact      => 'saml2test@example.com',
+        error_url        => '/error',
         authnreq_signed  => '0',
         want_assertions_signed => '0',
         @_,
